@@ -40,7 +40,10 @@ static gboolean zenity_general_multiple;
 static gboolean zenity_general_editable;
 static gchar   *zenity_general_uri;
 static gboolean zenity_general_dialog_no_wrap;
+static gboolean zenity_general_dialog_no_markup;
 static gint     zenity_general_timeout_delay;
+static gchar   *zenity_general_ok_button;
+static gchar   *zenity_general_cancel_button;
 
 /* Calendar Dialog Options */
 static gboolean zenity_calendar_active;
@@ -92,12 +95,15 @@ static gboolean zenity_progress_no_cancel;
 
 /* Question Dialog Options */
 static gboolean zenity_question_active;
-static gchar   *zenity_question_ok_button;
-static gchar   *zenity_question_cancel_button;
 
 /* Text Dialog Options */
 static gboolean zenity_text_active;
 static gchar   *zenity_text_font;
+static gchar   *zenity_text_checkbox;
+#ifdef HAVE_WEBKITGTK
+static gboolean zenity_text_enable_html;
+static gchar   *zenity_text_url;
+#endif
 
 /* Warning Dialog Options */
 static gboolean zenity_warning_active;
@@ -181,6 +187,24 @@ static GOptionEntry general_options[] = {
     /* Timeout for closing the dialog */
     N_("TIMEOUT")
   },
+  {
+    "ok-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_ok_button,
+    N_("Sets the label of the Ok button"),
+    N_("TEXT")
+  },
+  {
+    "cancel-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_cancel_button,
+    N_("Sets the label of the Cancel button"),
+    N_("TEXT")
+  },
   {
     NULL
   }
@@ -317,6 +341,14 @@ static GOptionEntry error_options[] = {
     N_("Do not enable text wrapping"),
     NULL
   },
+  {
+    "no-markup",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_general_dialog_no_markup,
+    N_("Do not enable pango markup")
+  },
   { 
     NULL 
   } 
@@ -350,6 +382,14 @@ static GOptionEntry info_options[] = {
     N_("Do not enable text wrapping"),
     NULL
   },
+  {
+    "no-markup",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_general_dialog_no_markup,
+    N_("Do not enable pango markup")
+  },
   { 
     NULL 
   }
@@ -667,24 +707,6 @@ static GOptionEntry question_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
-  {
-    "ok-label",
-    '\0',
-    G_OPTION_FLAG_NOALIAS,
-    G_OPTION_ARG_STRING,
-    &zenity_question_ok_button,
-    N_("Sets the label of the Ok button"),
-    N_("TEXT")
-  },
-  {
-    "cancel-label",
-    '\0',
-    G_OPTION_FLAG_NOALIAS,
-    G_OPTION_ARG_STRING,
-    &zenity_question_cancel_button,
-    N_("Sets the label of the Cancel button"),
-    N_("TEXT")
-  },
   {
     "no-wrap",
     '\0',
@@ -694,6 +716,14 @@ static GOptionEntry question_options[] = {
     N_("Do not enable text wrapping"),
     NULL
   },
+  {
+    "no-markup",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_general_dialog_no_markup,
+    N_("Do not enable pango markup")
+  },
   { 
     NULL 
   }
@@ -736,6 +766,35 @@ static GOptionEntry text_options[] = {
     N_("Set the text font"),
     N_("TEXT")
   },
+  {
+    "checkbox",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_text_checkbox,
+    N_("Enable an I read and agree checkbox"),
+    N_("TEXT")
+  },
+#ifdef HAVE_WEBKITGTK
+  {
+    "html",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_text_enable_html,
+    N_("Enable html support"),
+    NULL
+  },
+  {
+    "url",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_text_url,
+    N_("Sets an url instead of a file. Only works if you use --html option"),
+    N_("URL")
+  },
+#endif
   { 
     NULL 
   }
@@ -769,6 +828,14 @@ static GOptionEntry warning_options[] = {
     N_("Do not enable text wrapping"),
     NULL
   },
+  {
+    "no-markup",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_general_dialog_no_markup,
+    N_("Do not enable pango markup")
+  },
   { 
     NULL 
   }
@@ -1040,6 +1107,10 @@ zenity_option_free (void) {
   if (zenity_general_uri)
     g_free (zenity_general_uri);
   g_free (zenity_general_separator);
+  if (zenity_general_ok_button)
+    g_free (zenity_general_ok_button);
+  if (zenity_general_cancel_button)
+    g_free (zenity_general_cancel_button);
 
   if (zenity_calendar_date_format)
     g_free (zenity_calendar_date_format);
@@ -1060,13 +1131,10 @@ zenity_option_free (void) {
   if (zenity_list_hide_column)
     g_free (zenity_list_hide_column);
 
-  if (zenity_question_ok_button)
-    g_free (zenity_question_ok_button);
-  if (zenity_question_cancel_button)
-    g_free (zenity_question_cancel_button);
-
   if (zenity_text_font)
     g_free (zenity_text_font);
+  if (zenity_text_checkbox)
+    g_free (zenity_text_checkbox);
 
   if (zenity_colorsel_color)
     g_free (zenity_colorsel_color);
@@ -1145,7 +1213,10 @@ zenity_general_pre_callback (GOptionContext *context,
   zenity_general_multiple = FALSE;
   zenity_general_editable = FALSE;
   zenity_general_uri = NULL;
+  zenity_general_ok_button = NULL;
+  zenity_general_cancel_button = NULL;
   zenity_general_dialog_no_wrap = FALSE;
+  zenity_general_dialog_no_markup = FALSE;
   zenity_general_timeout_delay = -1;
 
   return TRUE;
@@ -1281,7 +1352,11 @@ zenity_text_pre_callback (GOptionContext *context,
 {
   zenity_text_active = FALSE;
   zenity_text_font = NULL;
-
+  zenity_text_checkbox = NULL;
+#ifdef HAVE_WEBKITGTK
+  zenity_text_enable_html = FALSE;
+  zenity_text_url = NULL;
+#endif
   return TRUE;
 }
 
@@ -1374,7 +1449,9 @@ zenity_general_post_callback (GOptionContext *context,
   results->data->window_icon = zenity_general_window_icon;
   results->data->width = zenity_general_width;
   results->data->height = zenity_general_height;
-  results->data->timeout_delay=zenity_general_timeout_delay;
+  results->data->timeout_delay = zenity_general_timeout_delay;
+  results->data->ok_label = zenity_general_ok_button;
+  results->data->cancel_label = zenity_general_cancel_button;
   return TRUE;
 }
 
@@ -1404,6 +1481,7 @@ zenity_calendar_post_callback (GOptionContext *context,
     results->calendar_data->day = zenity_calendar_day;
     results->calendar_data->month = zenity_calendar_month;
     results->calendar_data->year = zenity_calendar_year;
+
     if (zenity_calendar_date_format)
       results->calendar_data->date_format = zenity_calendar_date_format;
     else
@@ -1467,6 +1545,7 @@ zenity_error_post_callback (GOptionContext *context,
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_ERROR; 
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
+    results->msg_data->no_markup = zenity_general_dialog_no_markup;
   }
     
   return TRUE;
@@ -1483,7 +1562,8 @@ zenity_info_post_callback (GOptionContext *context,
   if (results->mode == MODE_INFO) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_INFO; 
-    results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
+    results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
+    results->msg_data->no_markup = zenity_general_dialog_no_markup;
   }
   
   return TRUE;
@@ -1610,7 +1690,6 @@ zenity_progress_post_callback (GOptionContext *context,
 		               GError        **error)
 {
   zenity_option_set_dialog_mode (zenity_progress_active, MODE_PROGRESS);
-
   if (results->mode == MODE_PROGRESS) {
     results->progress_data->dialog_text = zenity_general_dialog_text;
     results->progress_data->pulsate = zenity_progress_pulsate;
@@ -1649,14 +1728,11 @@ zenity_question_post_callback (GOptionContext *context,
 		               GError        **error)
 {
   zenity_option_set_dialog_mode (zenity_question_active, MODE_QUESTION);
-
-
   if (results->mode == MODE_QUESTION) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_QUESTION;
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
-    results->msg_data->ok_label = zenity_question_ok_button;
-    results->msg_data->cancel_label = zenity_question_cancel_button;
+    results->msg_data->no_markup = zenity_general_dialog_no_markup;
   }
 
   return TRUE;
@@ -1675,12 +1751,16 @@ zenity_text_post_callback (GOptionContext *context,
     results->text_data->editable = zenity_general_editable;
     results->text_data->no_wrap = zenity_general_dialog_no_wrap;
     results->text_data->font = zenity_text_font;
+    results->text_data->checkbox = zenity_text_checkbox;
+#ifdef HAVE_WEBKITGTK
+    results->text_data->html = zenity_text_enable_html;
+    results->text_data->url = zenity_text_url;
+#endif
   } else {
-    if (zenity_text_font)
+    if (zenity_text_font) 
       zenity_option_error (zenity_option_get_name (text_options, &zenity_text_font),
                            ERROR_SUPPORT);
   }
-    
   return TRUE;
 }
 
@@ -1695,7 +1775,8 @@ zenity_warning_post_callback (GOptionContext *context,
   if (results->mode == MODE_WARNING) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_WARNING;
-    results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
+    results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
+    results->msg_data->no_markup = zenity_general_dialog_no_markup;
   }
 
   return TRUE;
@@ -2064,6 +2145,15 @@ zenity_option_parse (gint argc, gchar **argv)
     if (results->mode != MODE_FILE && results->mode != MODE_TEXTINFO)
       zenity_option_error (zenity_option_get_name (text_options, &zenity_general_uri), ERROR_SUPPORT);
   
+  if (zenity_general_ok_button)
+    if(results->mode == MODE_FILE)
+      zenity_option_error (zenity_option_get_name (general_options, &zenity_general_ok_button), ERROR_SUPPORT);
+  
+  if (zenity_general_cancel_button)
+    if(results->mode == MODE_FILE || results->mode == MODE_ERROR || results->mode == MODE_WARNING || results->mode == MODE_INFO)
+      zenity_option_error (zenity_option_get_name (general_options, &zenity_general_cancel_button), ERROR_SUPPORT);
+  
+
   if (zenity_general_dialog_no_wrap)
     if (results->mode != MODE_INFO && results->mode != MODE_ERROR && results->mode != MODE_QUESTION && results->mode != MODE_WARNING && results->mode != MODE_TEXTINFO)
       zenity_option_error (zenity_option_get_name (text_options, &zenity_general_dialog_no_wrap), ERROR_SUPPORT);
